unit TestDUnitAssert;

interface

uses
  TestFramework, DUnitAssert;

type
  // Test methods for TAssertTestCase
  TTestDUnitAssert = class(TAssertTestCase)
  public
    procedure DoFail(const aMethod: string);
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestAreEqualBool;
    procedure TestAreEqualClass;
    procedure TestAreEqualDate;
    procedure TestAreEqualEnum;
    procedure TestAreEqualFloat;
    procedure TestAreEqualInt;
    procedure TestAreEqualSetEnum;
    procedure TestAreEqualSetNum;
    procedure TestAreEqualString;
    procedure TestAreEqualVariant;
    procedure TestAreEqualWideString;
    procedure TestAreNotEqualBool;
    procedure TestAreNotEqualClass;
    procedure TestAreNotEqualDate;
    procedure TestAreNotEqualEnum;
    procedure TestAreNotEqualFloat;
    procedure TestAreNotEqualInt;
    procedure TestAreNotEqualSetEnum;
    procedure TestAreNotEqualSetNum;
    procedure TestAreNotEqualString;
    procedure TestAreNotEqualVariant;
    procedure TestAreNotEqualWideString;
    procedure TestAreNotSameIntf;
    procedure TestAreNotSameObj;
    procedure TestAreSameIntf;
    procedure TestAreSameObj;
    procedure TestFail;
    procedure TestIsFalse;
    procedure TestIsInstanceOf;
    procedure TestIsNotInstanceOf;
    procedure TestIsNotNullIntf;
    procedure TestIsNotNullObj;
    procedure TestIsNullIntf;
    procedure TestIsNullObj;
    procedure TestIsTrue;
    procedure TestSetToString;
  end;

implementation

uses SysUtils, Variants;

type
  // numeric sets
  TElement0_8 = 0..8;
  TSet0_8 = set of TElement0_8;
  TElement1_9 = 1..9;
  TSet1_9 = set of TElement1_9;
  TElement0_100 = 0..100;
  TSet0_100 = set of TElement0_100;
  TElement1_255 = 1..255;
  TSet1_255 = set of TElement1_255;


  // char sets
  TSetChar = set of Char;

  // enum sets
  TElement = (Ele1, Ele2, Ele3, Ele4, Ele5, Ele6, Ele7, Ele8, Ele9, Ele10);
  TSetElement = set of TElement;

  {$M+}
  TElementNums = (EleN1=1, EleN2, EleN3, EleN4, EleN5, EleN6, EleN7, EleN8, EleN9, EleN10);
  TSetElementNums = set of TElementNums;

  TTestAssert = class(TDUnitAssert)
  end;

procedure TTestDUnitAssert.DoFail(const aMethod: string);
begin
  Assert.Fail(aMethod+' should have failed');
end;

procedure TTestDUnitAssert.SetUp;
begin
end;

procedure TTestDUnitAssert.TearDown;
begin
end;

procedure TTestDUnitAssert.TestAreEqualBool;
var
  Value: Boolean;
begin
  Value := True;
  Assert.AreEqual(True, Value); // no error
  try
    Assert.AreEqual(False, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(Bool)');
end;

procedure TTestDUnitAssert.TestAreEqualClass;
var
  Value: TClass;
begin
  Value := TTestDUnitAssert;
  Assert.AreEqual(TTestDUnitAssert, Value); // no error
  try
    Assert.AreEqual(TObject, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(TClass)');
end;

procedure TTestDUnitAssert.TestAreEqualDate;
var
  Value: TDateTime;
begin
  Value := Now;
  Assert.AreEqual(Value, Value); // no error
  try
    Assert.AreEqual(Now+1, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(TDateTime)');
end;

procedure TTestDUnitAssert.TestAreEqualEnum;
var
  Value,Value2: TElement;
begin
  Value := Ele1;
  Value2 := Ele1;
  Assert.AreEqual(Value2, Value, TypeInfo(TElement)); // no error
  Value := Ele2;
  Value2 := Ele2;
  //Assert.AreEqual(Ele2, Value, TypeInfo(TElement)); doesn't compile?!
  Assert.AreEqual(Value2, Value, TypeInfo(TElement)); // no error
  try
    Value2 := Ele1;
    Assert.AreEqual(Value2, Value, TypeInfo(TElement));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(Enum)');
end;

procedure TTestDUnitAssert.TestAreEqualFloat;
var
  Value: Double;
begin
  Value := 1.234;
  Assert.AreEqual(Value, Value); // no error
  Assert.AreEqual(1.2345, Value, 0.001); // no error
  try
    Assert.AreEqual(1.233, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(Extended)');
end;

procedure TTestDUnitAssert.TestAreEqualInt;
var
  Value: Integer;
begin
  Value := 1234;
  Assert.AreEqual(1234, Value); // no error
  try
    Assert.AreEqual(1233, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(Int)');
end;

procedure TTestDUnitAssert.TestAreEqualSetEnum;
var
  Value,Value2: TSetElement;
  ValueN,ValueN2: TSetElementNums;
begin
  Value := [Ele1..Ele9];
  Value2 := [Ele1..Ele9];
  Assert.AreEqual(Value2, Value, TypeInfo(TSetElement)); // no error
  ValueN := [EleN1..EleN9];
  ValueN2 := [EleN1..EleN9];
  try
    Assert.AreEqual(ValueN2, ValueN, TypeInfo(TSetElementNums)); // elements based on nums have no typeinfo!
    DoFail('AreEqual(SetEnum)');  // we can do the fail here within the try/except because it won't generate a DUnitException
  except
    on E: EDunitException do ;
  end;
  try
    Assert.AreEqual([Ele1..Ele10], Value, TypeInfo(TSetElement));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(SetEnum)');
end;

procedure TTestDUnitAssert.TestAreEqualSetNum;
var
  Value,Value2: TSet0_100;
begin
  Value := [1..99];
  Value2 := [1..99];
  Assert.AreEqual(Value2, Value, TypeInfo(TSet0_100)); // no error
  try
    Assert.AreEqual([#0..#255], Value, TypeInfo(TSet0_100));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(Set)');
end;

procedure TTestDUnitAssert.TestAreEqualString;
var
  Value: string;
begin
  Value := '';
  Assert.AreEqual('', Value); // no error
  Value := '1234';
  Assert.AreEqual('1234', Value); // no error
  try
    Assert.AreEqual('1233', Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(String)');
end;

procedure TTestDUnitAssert.TestAreEqualVariant;
var
  Value: Variant;
begin
  Assert.AreEqualVariant(Unassigned, Value); // no error
  Value := Null;
  Assert.AreEqualVariant(Null, Value); // no error
  Value := '';
  Assert.AreEqualVariant('', Value); // no error
  Value := '1234';
  Assert.AreEqualVariant('1234', Value); // no error
  Assert.AreEqualVariant(1234, Value); // no error
  Value := 1234;
  Assert.AreEqualVariant(1234, Value); // no error
  Assert.AreEqualVariant('1234', Value); // no error
  try
    Assert.AreEqualVariant('1233', Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqualVariant');
end;

procedure TTestDUnitAssert.TestAreEqualWideString;
var
  Value: WideString;
begin
  Value := '';
  Assert.AreEqual('', Value); // no error
  Value := '1234';
  Assert.AreEqual('1234', Value); // no error
  try
    Assert.AreEqual('1233', Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreEqual(WideString)');
end;

procedure TTestDUnitAssert.TestAreNotEqualBool;
var
  Value: Boolean;
begin
  Value := False;
  Assert.AreNotEqual(True, Value); // no error
  try
    Assert.AreNotEqual(False, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(Bool)');
end;

procedure TTestDUnitAssert.TestAreNotEqualClass;
var
  Value: TClass;
begin
  Value := TTestDUnitAssert;
  Assert.AreNotEqual(TObject, Value); // no error
  try
    Assert.AreNotEqual(TTestDUnitAssert, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(TClass)');
end;

procedure TTestDUnitAssert.TestAreNotEqualDate;
var
  Value: TDateTime;
begin
  Value := Now;
  Assert.AreNotEqual(Now+1, Value); // no error
  try
    Assert.AreNotEqual(Value, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(TDateTime)');
end;

procedure TTestDUnitAssert.TestAreNotEqualEnum;
var
  Value,Value2: TElement;
begin
  Value := Ele1;
  Value2 := Ele2;
  Assert.AreNotEqual(Value2, Value, TypeInfo(TElement)); // no error
  //Assert.AreEqual(Ele2, Value, TypeInfo(TElement)); doesn't compile?!
  Assert.AreNotEqual(Value, Value2, TypeInfo(TElement)); // no error
  try
    Value2 := Value;
    Assert.AreNotEqual(Value2, Value, TypeInfo(TElement));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(Enum)');
end;

procedure TTestDUnitAssert.TestAreNotEqualFloat;
  procedure FailCaseAreNotEqual(aExpected, aActual: Extended; aDelta: Extended=0);
  begin
    try
      if aDelta = 0 then
        Assert.AreNotEqual(aExpected, aActual)
      else
        Assert.AreNotEqual(aExpected, aActual, aDelta);
    except
      on E: ETestFailure do Exit;
    end;
    DoFail('AreNotEqual(Extended)');
  end;
var
  Value: Double;
begin
  Value := 1.234;
  Assert.AreNotEqual(1.233, Value);
  FailCaseAreNotEqual(Value, Value);
  FailCaseAreNotEqual(1.2345, Value, 0.001);
end;

procedure TTestDUnitAssert.TestAreNotEqualInt;
var
  Value: Integer;
begin
  Value := 1234;
  Assert.AreNotEqual(1233, Value); // no error
  try
    Assert.AreNotEqual(1234, Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(Int)');
end;

procedure TTestDUnitAssert.TestAreNotEqualSetEnum;
var
  Value,Value2: TSetElement;
  ValueN,ValueN2: TSetElementNums;
begin
  Value := [Ele1..Ele9];
  Value2 := [Ele1..Ele8];
  Assert.AreNotEqual(Value2, Value, TypeInfo(TSetElement)); // no error
  ValueN := [EleN1..EleN9];
  ValueN2 := [EleN1..EleN9];
  try
    Assert.AreNotEqual(ValueN2, ValueN, TypeInfo(TSetElementNums)); // elements based on nums have no typeinfo!
    DoFail('AreNotEqual(SetEnum)');  // we can do the fail here within the try/except because it won't generate a DUnitException
  except
    on E: EDunitException do ;
  end;
  try
    Assert.AreNotEqual([Ele1..Ele9], Value, TypeInfo(TSetElement));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(SetEnum)');
end;

procedure TTestDUnitAssert.TestAreNotEqualSetNum;
var
  Value,Value2: TSet0_100;
begin
  Value := [1..99];
  Value2 := [0..99];
  Assert.AreNotEqual(Value2, Value, TypeInfo(TSet0_100)); // no error
  try
    Assert.AreNotEqual([#1..#99], Value, TypeInfo(TSet0_100));
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqual(Set)');
end;

procedure TTestDUnitAssert.TestAreNotEqualString;
var
  Value: string;
  procedure FailCaseAreNotEqual(aExpected, aActual: string);
  begin
    try
      Assert.AreNotEqual(aExpected, aActual);
    except
      on E: ETestFailure do Exit;
    end;
    DoFail('AreNotEqual(String)');
  end;

begin
  Value := '';
  FailCaseAreNotEqual('', Value);
  Value := '1234';
  FailCaseAreNotEqual('1234', Value);
  Assert.AreNotEqual('1233', Value); // no error
end;

procedure TTestDUnitAssert.TestAreNotEqualVariant;
var
  Value: Variant;
begin
  Assert.AreNotEqualVariant(Null, Value); // no error
  Value := Null;
  Assert.AreNotEqualVariant(Unassigned, Value); // no error
  Assert.AreNotEqualVariant('', Value); // no error
  Value := 1234;
  Assert.AreNotEqualVariant('1233', Value); // no error
  try
    Assert.AreNotEqualVariant('1234', Value);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotEqualVariant');
end;

procedure TTestDUnitAssert.TestAreNotEqualWideString;
var
  Value: WideString;
  procedure FailCaseAreNotEqual(aExpected, aActual: WideString);
  begin
    try
      Assert.AreNotEqual(aExpected, aActual);
    except
      on E: ETestFailure do Exit;
    end;
    DoFail('AreNotEqual(WideString)');
  end;

begin
  Value := '';
  FailCaseAreNotEqual('', Value);
  Value := '1234';
  FailCaseAreNotEqual('1234', Value);
  Assert.AreNotEqual('1233', Value); // no error
end;

procedure TTestDUnitAssert.TestAreNotSameIntf;
var
  TestIntf: IInterface;
  TestIntf2: IInterface;
begin
  TestIntf := TInterfacedObject.Create;
  TestIntf2 := nil;
  Assert.AreNotSame(TestIntf2, TestIntf); // using nil iso TestIntf2 gives internal error!
  TestIntf2 := TInterfacedObject.Create;
  Assert.AreNotSame(TestIntf, TestIntf2);
  try
    Assert.AreNotSame(TestIntf, TestIntf);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreNotSame(intf)');  ;
end;

procedure TTestDUnitAssert.TestAreNotSameObj;
var
  TestObject: TObject;
  TestObject2: TObject;
begin
  TestObject := nil;
  TestObject2 := TObject.Create;
  try
    TestObject := TObject.Create;
    Assert.AreNotSame(nil, TestObject);
    Assert.AreNotSame(TestObject, TestObject2);
    try
      Assert.AreNotSame(TestObject, TestObject);
    except
      on E: ETestFailure do Exit;
    end;
  finally
    TestObject.Free;
    TestObject2.Free;
  end;
  DoFail('AreNotSame(object)');
end;

procedure TTestDUnitAssert.TestAreSameIntf;
var
  TestIntf: IInterface;
  TestIntf2: IInterface;
begin
  TestIntf := nil;
  Assert.AreSame(nil, TestIntf);
  TestIntf := TInterfacedObject.Create;
  Assert.AreSame(TestIntf, TestIntf);
  try
    TestIntf2 := TInterfacedObject.Create;
    Assert.AreSame(TestIntf2, TestIntf);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('AreSame(intf)');
end;

procedure TTestDUnitAssert.TestAreSameObj;
var
  TestObject: TObject;
  TestObject2: TObject;
begin
  TestObject := nil;
  Assert.AreSame(nil, TestObject);
  TestObject2 := TObject.Create;
  try
    TestObject := TObject.Create;
    Assert.AreSame(TestObject, TestObject);
    try
      Assert.AreSame(TestObject2, TestObject);
    except
      on E: ETestFailure do Exit;
    end;
  finally
    TestObject.Free;
    TestObject2.Free;
  end;
  DoFail('AreSame(object)');
end;

procedure TTestDUnitAssert.TestFail;
begin
  try
    Assert.Fail('Test');
  except
    on E: ETestFailure do Exit;
  end;
  Fail('.Fail failed');// check working of Fail with DUnit Fail method
end;

procedure TTestDUnitAssert.TestIsFalse;
begin
  Assert.IsFalse(False); // no error
  try
    Assert.IsFalse(True);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsFalse');
end;

procedure TTestDUnitAssert.TestIsInstanceOf;
begin
  Assert.IsInstanceOfType(TObject, Self); // no error
  Assert.IsInstanceOfType(TTestDUnitAssert, Self); // no error
  try
    Assert.IsInstanceOfType(TTestDUnitAssert, Self.Assert);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsInstanceOfType');
end;

procedure TTestDUnitAssert.TestIsNotInstanceOf;
  procedure FailCaseIsNotInstanceOf(aClass: TClass; aObject: TObject);
  begin
    try
      Assert.IsNotInstanceOfType(aClass, aObject);
    except
      on E: ETestFailure do Exit;
    end;
    DoFail('IsInstanceOfType');
  end;
begin
  Assert.IsNotInstanceOfType(TTestDUnitAssert, Self.Assert); // no error
  FailCaseIsNotInstanceOf(TTestDUnitAssert, Self);
  FailCaseIsNotInstanceOf(TObject, Self);
end;

procedure TTestDUnitAssert.TestIsNotNullIntf;
var
  TestIntf: IInterface;
begin
  TestIntf := TInterfacedObject.Create;
  Assert.IsNotNull(TestIntf);
  try
    TestIntf := nil;
    Assert.IsNotNull(TestIntf);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsNotNull(intf)');
end;

procedure TTestDUnitAssert.TestIsNotNullObj;
var
  TestObject: TObject;
begin
  TestObject := TObject.Create;
  try
    Assert.IsNotNull(TestObject);
  finally
    TestObject.Free;
  end;
  try
    TestObject := nil;
    Assert.IsNotNull(TestObject);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsNotNull(object)');
end;

procedure TTestDUnitAssert.TestIsNullIntf;
var
  TestIntf: IInterface;
begin
  TestIntf := nil;
  Assert.IsNull(TestIntf);
  try
    TestIntf := TInterfacedObject.Create;
    Assert.IsNull(TestIntf);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsNull(intf)');
end;

procedure TTestDUnitAssert.TestIsNullObj;
var
  TestObject: TObject;
begin
  TestObject := nil;
  Assert.IsNull(TestObject);
  try
    TestObject := TObject.Create;
    try
      Assert.IsNull(TestObject);
    finally
      TestObject.Free;
    end;
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsNull(object)');
end;

procedure TTestDUnitAssert.TestIsTrue;
begin
  Assert.IsTrue(True); // no error
  try
    Assert.IsTrue(False);
  except
    on E: ETestFailure do Exit;
  end;
  DoFail('IsTrue'); // DoFail should be outside try/except block!
end;

procedure TTestDUnitAssert.TestSetToString;
var
  sValue: String;
  Set1_9: TSet1_9;
  Set1_255: TSet1_255;
  SetElement: TSetElement;
begin
  sValue := TTestAssert.SetToString([0..8], TypeInfo(TSet0_8));
  Assert.AreEqual('[0,1,2,3,4,5,6,7,8]', sValue);
  sValue := TTestAssert.SetToString([1..9], TypeInfo(TSet0_8));
  Assert.AreEqual('[1,2,3,4,5,6,7,8]', sValue);

  sValue := TTestAssert.SetToString([0..8], TypeInfo(TSet1_9));
  Assert.AreEqual('[1,2,3,4,5,6,7,8]', sValue);
  Set1_9 := [0..9]; // this is allowed!?
  sValue := TTestAssert.SetToString(Set1_9, TypeInfo(TSet1_9));
  Assert.AreEqual('[1,2,3,4,5,6,7,8,9]', sValue);

  Set1_255 := [0..8];
  sValue := TTestAssert.SetToString(Set1_255, TypeInfo(TSet1_255));
  Assert.AreEqual('[1,2,3,4,5,6,7,8]', sValue);
  Set1_255 := [0..8];
  sValue := TTestAssert.SetToString(Set1_255, TypeInfo(TSet1_255));
  Assert.AreEqual('[1,2,3,4,5,6,7,8]', sValue);

  Set1_255 := [1,100,253..255];
  sValue := TTestAssert.SetToString(Set1_255, TypeInfo(TSet1_255));
  Assert.AreEqual('[1,100,253,254,255]', sValue);

  SetElement := [Ele1, Ele4, Ele8];
  sValue := TTestAssert.SetToString(SetElement, TypeInfo(TSetElement));
  Assert.AreEqual('[Ele1,Ele4,Ele8]', sValue);
end;

initialization
  // Register any test cases with the test runner
  RegisterTest(TTestDUnitAssert.Suite);
end.

